import logging
from typing import List

from .assets import Assets
from .index import Index


class Writer:
    def __init__(self, assets: Assets, title, parent_dir, filename, size, modified):
        self.__icon_folder = assets.icon_folder()

        self.__index_template = assets.index_template() \
            .read_text() \
            .replace("{TITLE}", title) \
            .replace("{PARENT}", parent_dir) \
            .replace("{FILENAME}", filename) \
            .replace("{SIZE}", size) \
            .replace("{MODIFIED}", modified)
        self.__file_template = assets.file_template().read_text()

    def write(self, indexes: List[Index]):
        for index in indexes:
            path = index.path.joinpath('index.html')
            logging.info(f"Writing {path}")

            html = index.to_html(self.__index_template, self.__file_template)

            path.write_text(html)

    def write_deep(self, indexes: List[Index]):
        if len(indexes) <= 0:
            return

        self.write(indexes)

        for index in indexes:
            self.write_deep(index.list_sub_indexes())



import os
from pathlib import Path
from typing import List

from . import reader, icon_manager
from .file import File
from .icon import icon_folder, Icon
from .reader import check_invalid


class Index:
    def __init__(self, path: Path, files: List[File]):
        self.path = path
        self.files = files
        self.icon_folder = path.joinpath(icon_folder)

    @classmethod
    def read_from_path(cls, path: Path):
        return cls(path, reader.list_files(path))

    def list_sub_indexes(self):
        sub_indexes = []

        for directory in self.path.iterdir():
            if directory.is_dir():
                if check_invalid(directory):
                    continue
                sub_indexes.append(Index.read_from_path(directory))
        return sub_indexes

    def icon(self, icon: Icon):
        icon_path = self.icon_folder.joinpath(icon.name)

        icon.copy_to(icon_path)

    def files_to_html(self, template: str) -> str:
        strings = []

        # create the icon folder
        if not self.icon_folder.exists():
            os.makedirs(self.icon_folder)

        # copy the go up icon
        self.icon(icon_manager.go_up_icon)

        self.files.sort(key=lambda f: f.path.name)

        for file in self.files:
            strings.append(file.to_html(template))

            self.icon(file.icon)

        return ''.join(strings)

    def to_html(self, index_template: str, file_template: str) -> str:
        return index_template.replace('#DIR', self.path.as_posix()) \
            .replace('#TITLE', self.path.name) \
            .replace('#FILES', self.files_to_html(file_template))



import time
from pathlib import Path

from . import byte_translate, icon_manager, url
from .icon import Icon


class File:
    def __init__(self, path: Path):
        if path.suffix == 'url':
            self.path = path.stem
            self.is_url = True
            self.url = url.get_url_from_path(path)
        else:
            self.path = path
            self.is_url = False

    def __str__(self):
        return self.path.__str__()

    @property
    def is_dir(self) -> bool:
        return self.path.is_dir()

    @property
    def extension(self) -> str:
        if self.is_dir:
            return '__folder'
        return self.path.suffix[1:]

    @property
    def filename(self) -> str:
        return self.path.name

    @property
    def icon(self) -> Icon:
        return icon_manager.lookup(self.extension)

    @property
    def size(self) -> str:
        if self.is_url:
            size = 0
        elif self.is_dir:
            size = sum(f.stat().st_size for f in self.path.rglob('*') if f.is_file())
        else:
            size = self.path.stat().st_size

        return byte_translate.translate(size)

    @property
    def modified(self):
        if self.is_url:
            return '-'

        return time.ctime(self.path.stat().st_mtime)

    def __replace_filename(self, template: str, key: str) -> str:
        if self.is_url:
            return template.replace(key, self.url)
        else:
            return template.replace(key, self.filename)

    def to_html(self, template: str) -> str:
        return self.__replace_filename(template, '#FILENAME') \
            .replace('#ICON', self.icon.relative_path.as_posix()) \
            .replace('#SIZE', self.size) \
            .replace('#MODIFIED', self.modified)



from pathlib import Path


class Assets:
    def __init__(self, path: Path):
        self.path = path

    def icon_folder(self):
        return self.path.joinpath('__icons')

    def icon_setting(self):
        return self.path.joinpath('icons.yml')

    def index_template(self):
        return self.path.joinpath('__index_template.html')

    def file_template(self):
        return self.path.joinpath('__file_template.html')



import math

__kb = 1024
__mb = __kb * 1024
__gb = __mb * 1024


def __rounding_div(a, b) -> float:
    return math.floor(a / b * 100) / 100


def translate(count: int) -> str:
    if count is None:
        return "-"

    if count >= __gb:
        return f"{__rounding_div(count, __gb)} GB"
    elif count >= __mb:
        return f"{__rounding_div(count, __mb)} MB"
    elif count >= __kb:
        return f"{__rounding_div(count, __kb)} KB"
    elif count > 0:
        return f"{count} B"
    else:
        return "-"



import argparse
from pathlib import Path

from .util import icon_manager
from .util.assets import Assets
from .util.index import Index
from .util.writer import Writer


def main():
    parser = argparse.ArgumentParser(description='Generates index.html files for file indexing.')

    parser.add_argument(
        'path', type=str,
        help='the path to the directory you want to index'
    )
    parser.add_argument(
        '--title', type=str,
        default="Index of #DIR",
        help='change the title of the index. (`#DIR`: the directory path)'
    )
    parser.add_argument(
        '--parent_dir', type=str,
        default="parent directory",
        help='change the name of the "parent directory" button in the index. (default: "parent directory")'
    )
    parser.add_argument(
        '--filename', type=str,
        default="File Name",
        help='change the name of the field "File Name" in the table. (default: "File Name")'
    )
    parser.add_argument(
        '--size', type=str,
        default="Size",
        help='change the name of the field "Size" in the table. (default: "Size")'
    )
    parser.add_argument(
        '--modified', type=str,
        default="Size",
        help='change the name of the field "Last Modified" in the table. (default: "Last Modified")'
    )

    args = parser.parse_args()

    # init
    root = Path(__file__).parent
    assets = Assets(root.joinpath('assets'))

    icon_manager.read_icons(assets)

    # actual program
    path = Path(args.path)
    index = Index.read_from_path(path)
    writer = Writer(
        assets,
        args.title, args.parent_dir, args.filename, args.size, args.modified
    )
    writer.write_deep([index])


if __name__ == "__main__":
    main()



import logging
from pathlib import Path
from shutil import copyfile
from typing import List, Optional

import yaml

from .assets import Assets

icon_folder = Path().joinpath('__icons')


class Icon:
    def __init__(self, source: Path, name: str, extensions: List[str]):
        self.source = source
        self.name = name
        self.extensions = extensions

    def __str__(self):
        return f"Icon(source={self.source}, name={self.name}, extensions={self.extensions})"

    @property
    def relative_path(self):
        return icon_folder.joinpath(self.name)

    def copy_to(self, destination: Path):
        if not destination.exists():
            logging.info(f"Copying Icon {destination}")
            copyfile(self.source, destination)


class IconManager:
    icons = []

    def read_icons(self, assets: Assets):
        with open(assets.icon_setting(), 'r') as stream:
            try:
                file = yaml.safe_load(stream)
                names = file.keys()

                for name in names:
                    extensions = file.get(name).get('extensions').split(',')
                    source = assets.icon_folder().joinpath(name)

                    icon = Icon(source, name, extensions)
                    logging.info(f"Read {icon}")

                    self.icons.append(icon)
            except yaml.YAMLError as e:
                print(e)

    def lookup(self, extension: str) -> Optional[Icon]:
        for icon in self.icons:
            if extension in icon.extensions:
                return icon
        return self.lookup('__unknown')

    @property
    def go_up_icon(self):
        return self.lookup('__go-up')



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="html-index",
    version="1.0.5",
    author="Madeline Sparkle",
    author_email="muguang138@gmail.com",
    description="HTML file indexing generator.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Mg138/html-index",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    entry_points={
        'console_scripts': [
            'index = html_index.index:main'
        ]
    },
    python_requires=">=3.6",
    include_package_data=True,
    install_requires=[
        'PyYAML~=5.4.1'
    ]
)



import shutil
from pathlib import Path
from typing import List

from .file import File
import logging


def check_invalid(path: Path):
    name = path.name
    return name.startswith('.') or name.startswith('__')


def list_files(path: Path) -> List[File]:
    files = []

    for file in path.glob('*'):
        if file.name in ['index.html', '.icons']:
            if file.is_dir():
                shutil.rmtree(file)
            else:
                file.unlink()
            continue

        if check_invalid(path):
            continue

        f = File(file)
        logging.info(f"Read File: {f}")
        files.append(f)

    return files



import os
from pathlib import Path

import yaml


def main():
    path = Path('__icons')
    data = {}

    for file in path.glob('*'):
        name = file.name
        extension = os.path.splitext(name)[0]
        data[name] = {'extensions': extension}

    with open('generated_icons.yml', 'w') as stream:
        yaml.dump(data, stream, )


if __name__ == '__main__':
    main()
